@@ -5,6 +5,8 @@ module Network.DNS.Internal where
 import Control.Exception (Exception)
 import Data.ByteString (ByteString)
 import qualified Data.ByteString.Char8 as BS
+import qualified Data.ByteString.Builder as L
+import qualified Data.ByteString.Lazy as L
 import Data.IP (IP, IPv4, IPv6)
 import Data.Maybe (fromMaybe)
 import Data.Typeable (Typeable)
@@ -28,8 +30,20 @@ data TYPE = A
           | SRV
           | DNAME
           | OPT
+          | DS
+          | RRSIG
+          | NSEC
+          | DNSKEY
+          | NSEC3
+          | NSEC3PARAM
+          | TLSA
+          | CDS
+          | CDNSKEY
+          | CSYNC
           | UNKNOWN Int deriving (Eq, Show, Read)
 
+-- https://www.iana.org/assignments/dns-parameters/dns-parameters.xhtml#dns-parameters-4
+--
 rrDB :: [(TYPE, Int)]
 rrDB = [
     (A,      1)
@@ -41,8 +55,18 @@ rrDB = [
   , (TXT,   16)
   , (AAAA,  28)
   , (SRV,   33)
-  , (DNAME, 39) -- RFC 2672
+  , (DNAME, 39) -- RFC 6672
   , (OPT,   41) -- RFC 6891
+  , (DS,    43) -- RFC 4034
+  , (RRSIG, 46) -- RFC 4034
+  , (NSEC,  47) -- RFC 4034
+  , (DNSKEY, 48) -- RFC 4034
+  , (NSEC3, 40) -- RFC 5155
+  , (NSEC3PARAM, 51) -- RFC 5155
+  , (TLSA,  52) -- RFC 6698
+  , (CDS,   59) -- RFC 7344
+  , (CDNSKEY, 60) -- RFC 7344
+  , (CSYNC, 62) -- RFC 7477
   ]
 
 data OPTTYPE = ClientSubnet
@@ -137,6 +161,7 @@ data DNSFlags = DNSFlags {
   , recDesired   :: Bool
   , recAvailable :: Bool
   , rcode        :: RCODE
+  , authenData   :: Bool
   } deriving (Eq, Show)
 
 ----------------------------------------------------------------
@@ -157,7 +182,7 @@ data RCODE
   | NotImpl
   | Refused
   | BadOpt
-  deriving (Eq, Show, Enum, Bounded)
+  deriving (Eq, Ord, Show, Enum, Bounded)
 
 ----------------------------------------------------------------
 
@@ -201,11 +226,12 @@ data RData = RD_NS Domain
            | RD_SRV Int Int Int Domain
            | RD_OPT [OData]
            | RD_OTH ByteString
-    deriving (Eq)
+           | RD_TLSA Int Int Int ByteString
+    deriving (Eq, Ord)
 
 instance Show RData where
   show (RD_NS dom) = BS.unpack dom
-  show (RD_MX prf dom) = BS.unpack dom ++ " " ++ show prf
+  show (RD_MX prf dom) = show prf ++ " " ++ BS.unpack dom
   show (RD_CNAME dom) = BS.unpack dom
   show (RD_DNAME dom) = BS.unpack dom
   show (RD_A a) = show a
@@ -216,11 +242,12 @@ instance Show RData where
   show (RD_SRV pri wei prt dom) = show pri ++ " " ++ show wei ++ " " ++ show prt ++ BS.unpack dom
   show (RD_OPT od) = show od
   show (RD_OTH is) = show is
+  show (RD_TLSA use sel mtype dgst) = show use ++ " " ++ show sel ++ " " ++ show mtype ++ " " ++ (BS.unpack $ L.toStrict . L.toLazyByteString . L.byteStringHex $ dgst)
 
 
 data OData = OD_ClientSubnet Int Int IP
            | OD_Unknown Int ByteString
-    deriving (Eq,Show)
+    deriving (Eq,Show,Ord)
 
 ----------------------------------------------------------------
 
@@ -236,6 +263,7 @@ defaultQuery = DNSMessage {
          , recDesired   = True
          , recAvailable = False
          , rcode        = NoErr
+         , authenData   = False
          }
      }
   , question   = []
@@ -254,6 +282,7 @@ defaultResponse =
               qOrR = QR_Response
             , authAnswer = True
             , recAvailable = True
+            , authenData = False
             }
         }
       }
