@@ -14,6 +14,7 @@ module Network.DNS.Resolver (
   , lookupAuth
   -- ** Raw looking up function
   , lookupRaw
+  , lookupRawAD
   , fromDNSMessage
   , fromDNSFormat
   ) where
@@ -27,7 +28,7 @@ import Network.DNS.Decode
 import Network.DNS.Encode
 import Network.DNS.Internal
 import qualified Data.ByteString.Char8 as BS
-import Network.Socket (HostName, Socket, SocketType(Datagram), sClose, socket, connect)
+import Network.Socket (HostName, Socket, SocketType(Datagram), close, socket, connect)
 import Network.Socket (AddrInfoFlag(..), AddrInfo(..), SockAddr(..), PortNumber(..), defaultHints, getAddrInfo)
 import Prelude hiding (lookup)
 import System.Random (getStdRandom, randomR)
@@ -166,7 +167,7 @@ makeAddrInfo addr mport = do
 --   'Resolver'. If multiple 'Resolver's are necessary for
 --   concurrent purpose, use 'withResolvers'.
 withResolver :: ResolvSeed -> (Resolver -> IO a) -> IO a
-withResolver seed func = bracket (openSocket seed) sClose $ \sock -> do
+withResolver seed func = bracket (openSocket seed) close $ \sock -> do
     connectSocket sock seed
     func $ makeResolver seed sock
 
@@ -181,7 +182,7 @@ withResolvers seeds func = bracket openSockets closeSockets $ \socks -> do
     func resolvs
   where
     openSockets = mapM openSocket seeds
-    closeSockets = mapM sClose
+    closeSockets = mapM close
 
 openSocket :: ResolvSeed -> IO Socket
 openSocket seed = socket (addrFamily ai) (addrSocketType ai) (addrProtocol ai)
@@ -291,7 +292,9 @@ lookupAuth = lookupSection authority
 --                                      trunCation = False,
 --                                      recDesired = True,
 --                                      recAvailable = True,
---                                      rcode = NoErr },
+--                                      rcode = NoErr,
+--                                      authenData = False
+--                                    },
 --                        },
 --             question = [Question { qname = \"www.example.com.\",
 --                                    qtype = A}],
@@ -305,19 +308,25 @@ lookupAuth = lookupSection authority
 --  @
 --
 lookupRaw :: Resolver -> Domain -> TYPE -> IO (Either DNSError DNSMessage)
-lookupRaw = lookupRawInternal receive
+lookupRaw = lookupRawInternal receive False
+
+lookupRawAD :: Resolver -> Domain -> TYPE -> IO (Either DNSError DNSMessage)
+lookupRawAD = lookupRawInternal receive True
+
+
 
 lookupRawInternal ::
     (Socket -> IO DNSMessage)
+    -> Bool
     -> Resolver
     -> Domain
     -> TYPE
     -> IO (Either DNSError DNSMessage)
-lookupRawInternal _ _   dom _
+lookupRawInternal _ _ _   dom _
   | isIllegal dom     = return $ Left IllegalDomain
-lookupRawInternal rcv rlv dom typ = do
+lookupRawInternal rcv ad rlv dom typ = do
     seqno <- genId rlv
-    let query = composeQuery seqno [q]
+    let query = (if ad then composeQueryAD else composeQuery) seqno [q]
         checkSeqno = check seqno
     loop query checkSeqno 0 False
   where
